/*
Copyright (C) 2000  Felipe Cerqueira dos Santos

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

fcerqueira@bufferoverflow.org

Version: 2.2 Date: Date: Fri May  5 11:40:43 2000

*/


#include <sys/types.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <syslog.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <wait.h>
#include <time.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include "lstat.h"

/* Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
rights reserved. */
#include "global.h"
#include "md5.h"
/* EOF */

#define config_temp "/tmp/lstatconf.tmp"
//#define CONFIG      "/etc/lstatconf"

#define TIME_STRING_BUF 50

#define banner1 "                     LSTAT V. 2.3.1 - QUICK FILE CHECKER"
#define banner2 " 		  Felipe Cerqueira - www.bufferoverflow.org"

/* MD5 */
#define TEST_BLOCK_LEN 1000
#define TEST_BLOCK_COUNT 1000


#define MD_CTX MD5_CTX
#define MDInit MD5Init
#define MDUpdate MD5Update
#define MDFinal MD5Final
/* MD5 EOF */


int create_temp_file(void);
void open_config(void);
void list_path(char *path, FILE * output);
void file_stats(char *);
char *TimeString(time_t, char *);
void sniffer_check();
extern int verify1();
extern int verify2();


/* Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
rights reserved. */
static void MDFile PROTO_LIST((char *));


char *DATABASE;

char buff = '\\';

unsigned char buffer[1024], digest[16];

unsigned int count = 0;

FILE *logfd;

/* Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
rights reserved. */
static void MDFile(filename)
char *filename;
{
    FILE *file;
    MD_CTX context;
    int len;


    if ((file = fopen(filename, "rb")) != NULL) {
	MDInit(&context);
	while ((len = fread(buffer, 1, 1024, file)))
	    MDUpdate(&context, buffer, len);

	MDFinal(digest, &context);

	fclose(file);

    }
}



void file_stats(char *file)
{
    FILE *fp;

    struct stat statbuf;

    char timebuf[TIME_STRING_BUF]; //verify[12];

    register int i;

    umask(0177);

    if ((fp = fopen(DATABASE, "a+")) == NULL) {
	fprintf(stderr, "Cannot access %s: %s\n", DATABASE,
		strerror(errno));
	exit(1);
    }


    if (lstat(file, &statbuf)) {
	fprintf(stderr, "Cannot verify %s: %s\n", file, strerror(errno));
	return;
    }


    count++;
    fprintf(stderr, "                                             \r");
    fprintf(stderr, "\033[5;0H");
    switch (buff) {
    case '\\':
	buff = '|';
	fprintf(stderr, "[%-58s] Total [%-5i] ( %c )\r", file, count,
		buff);
	break;
    case '|':
	buff = '/';
	fprintf(stderr, "[%-58s] Total [%-5i] ( %c )\r", file, count,
		buff);
	break;
    case '/':
	buff = '-';
	fprintf(stderr, "[%-58s] Total [%-5i] ( %c )\r", file, count,
		buff);
	break;
    case '-':
	buff = '\\';
	fprintf(stderr, "[%-58s] Total [%-5i] ( %c )\r", file, count,
		buff);
	break;
    }

    fprintf(fp, "%s|%i|%i|%i|%05o|%s|", file, statbuf.st_uid,
	    statbuf.st_gid, statbuf.st_size, statbuf.st_mode,
	    (char *) TimeString((int)statbuf.st_ctime, (char *) timebuf));

/* Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
rights reserved. */
    MDFile(file);

    for (i = 0; i < 16; i++)
	fprintf(fp, "%02x", digest[i]);

    fprintf(fp, "\n");

    /* EOF */

    fclose(fp);

}



char *TimeString(time_t t, char *buf)
{				/* START */
    struct tm *local;

    local = localtime(&t);

    strftime(buf, TIME_STRING_BUF, "%c", local);

    return buf;

}



void list_path(char *buf, FILE * output)
{
    DIR *dir;
    struct dirent *ent;
    struct stat p;

    char *path, *verify;

    int perm, pos;

    unsigned int i;

    if (!(dir = opendir(buf))) {
	fprintf(stderr, "** opendir() %s : %s\n", buf, strerror(errno));
	return;
    }

    while ((ent = readdir(dir))) {
	if (strcmp(ent->d_name, ".") == 0
	    || strcmp(ent->d_name, "..") == 0) continue;

	i = strlen(buf) + strlen(ent->d_name) + 1;

	path = malloc(i);

	snprintf(path, i, "%s%s", buf, ent->d_name);

	if (lstat(path, &p)) {
	    perror("lstat()");
	    continue;
	}

	verify = malloc(10);

	snprintf(verify, 10, "%05o", p.st_mode);

	perm = atoi(verify);

	if (perm >= 40000 && perm <= 47777) {
	    pos = ftell(output);
	    fseek(output, 0, SEEK_END);

	    fprintf(output, "2 %s/\n", path);

	    fseek(output, pos, SEEK_SET);

	} else
	    (void) file_stats(path);

	free(path);
	free(verify);
    }

    closedir(dir);
}



void open_config(void)
{
    FILE *input;
    char *buf, *file;
    unsigned short config_opt;

    buf = malloc(1024);
    file = malloc(1024);

    if ((input = fopen(config_temp, "r+")) == NULL) {
	fprintf(stderr, "Cannot read config file from %s : %s\n", config_temp, strerror(errno));
	fprintf(logfd, "Error: Cannot read config file from %s : %s\n", config_temp, strerror(errno));
	_exit(-1);
    }

    while (!feof(input)) {
	fscanf(input, "%[^\n]\n", buf);	/* Get paths */
	config_opt = atoi(strtok(buf, " "));
	sprintf(file, "%s", strtok(NULL, "\n"));

	if (strcmp(file, "/proc/") != 0 && strcmp(file, "/dev/") != 0
	    && (buf[0] != '#') && (buf[0] != '$') && ((buf[0] == '1' || buf[0] == '2'))) {
	    switch (config_opt) {
	    case 1:
		(void) file_stats(file);
		break;
	    case 2:
		(void) list_path(file, input);
		break;
	    }
	}
    }

    free(buf);
    free(file);

}


int create_temp_file(void)
{
    struct stat p;
    int fdin, fdout;
    char *buf;
    off_t len;

    if ((fdin = open(CONFIG, O_RDONLY)) < 0) {
	fprintf(stderr, "Cannot read from %s : %s\n", CONFIG, strerror(errno));
	fprintf(logfd, "Error : Cannot read from %s : %s\n", CONFIG, strerror(errno));
	return 1;
    }


    if ((fdout = open(config_temp, O_CREAT | O_WRONLY, 0400)) < 0) {
	perror("open()");
	fprintf(stderr, "Cannot read from %s : %s\n", config_temp, strerror(errno));
	fprintf(logfd, "Error : Cannot read from %s : %s\n", config_temp, strerror(errno));
	return 1;
    }

    if ((fstat(fdin, &p)) < 0) {
	perror("fstat()");
	return 1;
    }

    len = p.st_size;

    if ((buf = mmap(0, len, PROT_READ, MAP_SHARED, fdin, 0)) ==
	(void *) -1) {
	perror("mmap()");
	exit(0);
    }

    write(fdout, buf, len);

    munmap(buf, len);

    close(fdin);

    close(fdout);

    return 0;
}



int main(int argc, char **argv)
{
    struct stat p;
    struct timezone tz; 
    struct timeval tv; 
    time_t now;
	
    char *DatabaseMD5, *tempMD5;
    unsigned short FLAG;

    signal(SIGINT, SIG_IGN);
    signal(SIGHUP, SIG_IGN);

    fprintf(stdout, "\033[2J");
    fprintf(stdout, "\033[0;0H");
    
    gettimeofday(&tv, &tz);	
    now = tv.tv_sec;
 	
    puts(banner1);
    puts(banner2);

    if ((getuid()) != 0) {
        fprintf(stderr,
                "\n[ Voce precisa ser root para executar este programa ]\n");
        return 1;
    }


    if (( logfd = fopen(LSTATLOG, "a+")) == NULL) {
        fprintf(stderr, "Cannot read/write %s : %s\n", LSTATLOG, strerror(errno));
        return -1;
    }

    fprintf(logfd, "LSTAT IDS SYSTEM\nDate: %s\n", ctime(&now));
	
    if (argc > 1) {
	if ((strcmp(argv[1], "-update")) == 0)
	    remove(Database);
	    fprintf(logfd, "Warning: Database update\n");
    }


    remove(temp);
    remove(config_temp);

    openlog("LSTAT", LOG_PID, LOG_USER);
    syslog(LOG_WARNING, "LSTAT Start\n");

    if ((create_temp_file()) != 0) {
	fprintf(stderr, "Cant create a temp file %s from %s\n",
		config_temp, CONFIG);
	fprintf(logfd, "Error: Cannot creat a temporary file %s : %s\n", temp, strerror(errno));
	exit(1);
    }

    if ((DATABASE = malloc(strlen(Database))) == NULL) {
	perror("malloc()");
	return 1;
    }

    strncpy(DATABASE, Database, sizeof(Database));

    if (lstat(DATABASE, &p)) {
	/* Database not found */
	fprintf(stderr, "\n ** Creating Database **\n");
	syslog(LOG_WARNING, "Creating Database\n");
	if( (argc > 1) && (strcmp(argv[1], "-update") !=0 ) )
		fprintf(logfd, "\nDatabase not found, creating...\n");
	FLAG = 1;
    } else {
	free(DATABASE);
	syslog(LOG_WARNING, "Verifing System\n");
	fprintf(logfd, "\nDatabase found, verifing...\n");
	if ((DATABASE = malloc(sizeof(temp))) == NULL) {
	    perror("malloc()");
	    return 1;
	}

	strncpy(DATABASE, temp, sizeof(temp));
	fprintf(stderr, "\n ** Verification **\n");

	FLAG = 2;		/* run verify1 and verify2 */
    }

    (void) open_config();


    fprintf(stderr, "\nDone.\n");

    if (FLAG == 2) {
	fprintf(stderr, "\n ** Starting Verification **\n");
	/* verify databases */

	(void) MDFile(Database);
	if ((DatabaseMD5 = malloc(strlen(digest))) == NULL) {
	    perror("malloc()");
	    return 1;
	}
	strncpy(DatabaseMD5, digest, strlen(digest));

	(void) MDFile(temp);
	if ((tempMD5 = malloc(strlen(digest))) == NULL) {
	    perror("malloc()");
	    return 1;
	}
	strncpy(tempMD5, digest, strlen(digest));

	if (strncmp(DatabaseMD5, tempMD5, 16) == 0) {
	    fprintf(stderr,
		    " \033[1;32m**\033[0;37m System Intact! \033[1;32m**\033[0;37m\n");
	    free(DatabaseMD5);
	    free(tempMD5);
	    syslog(LOG_WARNING, "System Intact\n");
	    fprintf(logfd, "Okay, system Intact\n");	
	    goto fim;
	}

	fprintf(stderr,
		"\033[5;1;31m **\033[0;0;37m System Changed: \033[5;1;31m**\033[0;0;37m\n");
	syslog(LOG_WARNING, "System Changed!\n");
	fprintf(logfd, "Warning : System Changed:\n");
	free(DatabaseMD5);
	free(tempMD5);

	fflush(logfd);
	(int) verify1(); 
	(int) verify2(); 
    }

  fim:
    free(DATABASE);

    remove(temp);
    remove(config_temp);

    (void) sniffer_check();

    syslog(LOG_WARNING, "Exit.\n");
    fprintf(logfd, "\nTerminated\n\n\n");
    fprintf(stderr, "\rTerminated\n");
 
    fclose(logfd);	 	
    closelog();
    return 0;
}
