#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <crypt.h>
#include <errno.h>
#include <fcntl.h>

#include <netinet/in.h>

#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "lstat.h"

#define banner		"LSTAT log server version 1.0 ready\n\n"

#define auth1		"-OK Password incorrect\n"

#define LOG_MSG1	"-OK No logs found\n"
#define LOG_MSG2	"-OK Log file not found!!\n"

#define MAXLENGTH	70

#include "lstat.h"

int _socket, newsock;

char BUF[MAXLENGTH];

int verify_password(char *PASS);

int send_log(int newsoc);

void kill_lstat_server(); 

void kill_lstat_server() {
	fprintf(stdout, "\nLSTAT server going down !@#$\n");
	close(_socket);
	exit(EXIT_FAILURE);
}

int send_log(int newsock) {
	int fdin;
	char *logbuf;
	struct stat statbuf;
	off_t len;
	
	if (( fdin = open(LSTATLOG, O_RDWR)) < 0) {
		send(newsock, LOG_MSG2, strlen(LOG_MSG2), 0);
		return (EXIT_FAILURE);	
	}	

	if ((fstat(fdin, &statbuf)) < 0)
		exit(EXIT_FAILURE);
	
	len = statbuf.st_size;

	if( len == 0) {
		send(newsock, LOG_MSG1, strlen(LOG_MSG1), 0);
		return 0;
	}

	if (( logbuf = mmap(0, len, PROT_READ, MAP_SHARED, fdin, 0)) == (void *)-1)
		return (EXIT_FAILURE);

	send(newsock, logbuf, len, 0);

	close(fdin); munmap(logbuf, len);

	if (( fdin = open(LSTATLOG, O_TRUNC)) < 0)  // zerar o file	
		return (EXIT_FAILURE);

	close(fdin);
	return 0;
}
 
int verify_password(char *PASS) {
	FILE *fp;
	char cbuf[512];
	char password[20];
	char *pass;

	unsigned int size;

	static char saltChars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";

	char salt[3];

	if ( PASS == NULL ) {
		return 1; // password incorrect
	}

	size = strlen(PASS);

	salt[0] = saltChars[size / 2];
	salt[1] = saltChars[size / 2 + 1];
	salt[2] = 0;

	pass = (char *) crypt(PASS, salt);

	if (( fp = fopen (CONFIG, "r")) == NULL) {
		perror("fopen()");
		return 1; // cannot read LSTATLOG && password incorrect
	}
	
	while(!feof(fp)) {
		fscanf(fp, "%[^\n]\n", cbuf);
		if ( cbuf[0] == '$' ) {
			(void) strtok(cbuf, " ");
			sprintf(password, strtok(NULL, " "));

			if ( strncmp(password, pass, strlen(pass)) == 0) {
				fclose(fp); 
				return 0; // password okay
			} else {
				fclose(fp);
				return 1; // password incorrect
			}
		}
	}

        fclose(fp);
	return 1; // password incorrect
}


int main(int argc, char **argv)
{

    struct sockaddr_in sin;
    unsigned int size; 
    unsigned short status;
    int n;
       		
    signal(SIGHUP, SIG_IGN);
    signal(SIGTERM, kill_lstat_server);       
 	
    if ((_socket = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
	perror("socket()");
	 exit(-1);
    }

    fputs(banner, stdout); 

    if(fork() != 0) 
	exit(-1); 

    sin.sin_family = AF_INET;
    sin.sin_port = htons(atoi(PORT));
    sin.sin_addr.s_addr = INADDR_ANY;

    bzero(&(sin.sin_zero), 8);

    if (bind(_socket, (struct sockaddr *) &sin, sizeof(struct sockaddr))
	== -1) {
	perror("bind()");
	fprintf(stderr, "\nSolutions:\tkillall %s\n", argv[0]);
	fprintf(stderr, "\t\tWait any minutes and run again\n");
	exit(1);
    }

    if (listen(_socket, 0) < 0) {
	perror("listen()");
	exit(-1);
    }

    while (1) {

	size = sizeof(struct sockaddr_in);

	if ((newsock = accept(_socket, (struct sockaddr *) &sin, &size)) <
	    0) {
	    perror("accept()");	
	    continue;
	}

	while(( n = read(newsock, BUF, MAXLENGTH)) > 0 ) {

		BUF[strlen(BUF)] = '\0';

	        status = verify_password(BUF);

        	if (status == 1) {	  
		 	send(newsock, auth1, strlen(auth1), 0); // incorrect
			bzero(BUF, MAXLENGTH);
       			break;
		}

		if (status == 0) {
			bzero(BUF, MAXLENGTH);
			(void) send_log(newsock);
			break;
		}

		bzero(BUF, MAXLENGTH);

	}

	close(newsock);
    }
    return 0;
}
