/*
Copyright (C) 2000  Felipe Cerqueira dos Santos

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

fcerqueira@bufferoverflow.org
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <syslog.h>

#include "lstat.h"

#define PFILE Database
#define TFILE temp

#define TOTLINE  400 	

#define MAXBUFF  1024

struct lstat {
    char name[256];
    int uid;
    int gid;
    int size;
    int perm;
    char date[28];
    char md5msg[36];
};

struct lstat lstat_p[TOTLINE], *lptr, lstat_t;

char tbuffer[MAXBUFF];
int pos = 0;

void carregar(int pos);
int memview(); 			// return 0 if exist and 1 if not exist
void diskview();			

FILE *pfile, *tfile, *logfd;

void diskview() {
	struct lstat lstat_d; // from "Database"
	int pos_cur = 0; 
	char *buf;

	if ((buf = malloc(MAXBUFF)) == NULL) {
		fprintf(stderr, "cannot use malloc()\n");
		exit(EXIT_FAILURE);
	}

	bzero(buf, MAXBUFF);
	fseek(pfile, 0, SEEK_SET); 

	while((!feof(pfile)) && (pos_cur == 0)) {
		fscanf(pfile, "%[^\n]\n", buf);

		if ( strlen(buf) == 0 ) 
			continue;

		sprintf(lstat_d.name, "%s", strtok(buf, "|"));
		lstat_d.uid = atoi(strtok(NULL, "|"));
                lstat_d.gid = atoi(strtok(NULL, "|"));
                lstat_d.size = atoi(strtok(NULL, "|"));
                lstat_d.perm = atoi(strtok(NULL, "|"));
                sprintf(lstat_d.date, "%s", strtok(NULL, "|"));
                sprintf(lstat_d.md5msg, "%s", strtok(NULL, "|"));

		if ( strcmp(lstat_d.name, lstat_t.name) == 0) {
			pos_cur = ftell(pfile); // magic?!@# :]

			if(lstat_t.uid != lstat_d.uid) {
				fprintf(stderr, "\a=> %s - uid: %i -> %i\n", lstat_d.name, lstat_d.uid, lstat_t.uid);
				syslog(LOG_WARNING, "=> %s - uid: %i -> %i\n", lstat_d.name, lstat_d.uid, lstat_t.uid);
				fprintf(logfd, "=> %s - uid: %i -> %i\n", lstat_d.name, lstat_d.uid, lstat_t.uid);
			}

                        if(lstat_t.gid != lstat_d.gid) {
                                fprintf(stderr, "\a=> %s - gid: %i -> %i\n", lstat_d.name, lstat_d.gid, lstat_t.gid);
				syslog(LOG_WARNING, "=> %s - gid: %i -> %i\n", lstat_d.name, lstat_d.gid, lstat_t.gid);
				fprintf(logfd, "=> %s - gid: %i -> %i\n", lstat_d.name, lstat_d.gid, lstat_t.gid);
                        }
			
                        if(lstat_t.size != lstat_d.size) {
                                fprintf(stderr, "\a=> %s - size: %i -> %i\n", lstat_d.name, lstat_d.size, lstat_t.size);
				syslog(LOG_WARNING, "=> %s - size: %i -> %i\n", lstat_d.name, lstat_d.size, lstat_t.size);
				fprintf(logfd, "=> %s - size: %i -> %i\n",
lstat_d.name, lstat_d.size, lstat_t.size);
                        }

                        if(lstat_t.perm != lstat_d.perm) {
                                fprintf(stderr, "\a=> %s - permissions: %i -> %i\n", lstat_d.name, lstat_d.perm, lstat_t.perm);
				syslog(LOG_WARNING, "=> %s - permissions: %i -> %i\n", lstat_d.name, lstat_d.perm, lstat_t.perm);
				fprintf(logfd, "=> %s - permissions: %i -> %i\n", lstat_d.name, lstat_d.perm, lstat_t.perm);
                        }
  
			if(strcmp(lstat_d.date, lstat_t.date) != 0) {
				fprintf(stderr, "\a=> %s - last change: %s -> %s\n", lstat_d.name, lstat_d.date, lstat_t.date);
				syslog(LOG_WARNING, "=> %s - last change: %s -> %s\n", lstat_d.name, lstat_d.date, lstat_t.date);
				fprintf(logfd, "=> %s - last change: %s -> %s\n", lstat_d.name, lstat_d.date, lstat_t.date);
			}

                        if(strcmp(lstat_d.date, lstat_t.date) != 0) {
				fprintf(stderr, "\a=> %s - md5 digest: %s -> %s\n", lstat_d.name, lstat_d.md5msg, lstat_t.md5msg);
				syslog(LOG_WARNING, "=> %s - md5 digest: %s -> %s\n", lstat_d.name, lstat_d.md5msg, lstat_t.md5msg);
				fprintf(logfd, "=> %s - md5 digest: %s -> %s\n", lstat_d.name, lstat_d.md5msg, lstat_t.md5msg);
			}
			bzero(buf, MAXBUFF);
		}
	}
	if ( pos_cur == 0 ) {
		fprintf(stderr, "\a=> newfile : %s - uid: %i gid: %i permissions: %i md5 digest: %s\n", lstat_t.name, lstat_t.uid, lstat_t.gid, lstat_t.perm, lstat_t.md5msg);
		syslog(LOG_WARNING, "=> newfile : %s - uid: %i gid: %i permissions: %i md5 digest: %s\n", lstat_t.name, lstat_t.uid, lstat_t.gid, lstat_t.perm, lstat_t.md5msg);
		fprintf(logfd, "=> newfile : %s - uid: %i gid: %i permissions: %i md5 digest: %s\n", lstat_t.name, lstat_t.uid, lstat_t.gid, lstat_t.perm,
lstat_t.md5msg);
	} else
		(void) carregar(pos_cur);

	free(buf);
}	    	
											
int memview() {
	unsigned short ret = 1; int cnt;

	lptr = &lstat_p[0];

	for (cnt = 0; cnt < TOTLINE; cnt++) {
		if (strcmp(lptr->name, lstat_t.name) == 0) {
			ret = 0; // ret value when found

			if (lptr->uid != lstat_t.uid) {
				fprintf(stderr, "\a=> %s - uid: %i -> %i\n", lptr->name, lptr->uid, lstat_t.uid);
				syslog(LOG_WARNING, "=> %s - uid: %i -> %i\n", lptr->name, lptr->uid, lstat_t.uid);
				fprintf(logfd, "=> %s - uid: %i -> %i\n", lptr->name, lptr->uid, lstat_t.uid);
			}

			if (lptr->gid != lstat_t.gid) {
				fprintf(stderr, "\a=> %s - gid: %i -> %i\n", lptr->name, lptr->gid, lstat_t.gid);
				syslog(LOG_WARNING, "=> %s - gid: %i -> %i\n", lptr->name, lptr->gid, lstat_t.gid);
				fprintf(logfd, "=> %s - gid: %i -> %i\n", lptr->name, lptr->gid, lstat_t.gid);
			}

			if (lptr->size != lstat_t.size) {
				fprintf(stderr, "\a=> %s - size: %i -> %i\n", lptr->name, lptr->size, lstat_t.size);
				syslog(LOG_WARNING, "=> %s - size: %i -> %i\n", lptr->name, lptr->size, lstat_t.size);
				fprintf(logfd, "=> %s - size: %i -> %i\n",
lptr->name, lptr->size, lstat_t.size);
			}

			if (lptr->perm != lstat_t.perm) {
				fprintf(stderr, "\a=> %s - permissions: %i -> %i\n", lptr->name, lptr->perm, lstat_t.perm);
				syslog(LOG_WARNING, "=> %s - permissions: %i -> %i\n", lptr->name, lptr->perm, lstat_t.perm);		
				fprintf(logfd, "=> %s - permissions: %i -> %i\n", lptr->name, lptr->perm, lstat_t.perm);
			}

			if (strcmp(lptr->date, lstat_t.date) != 0) {
				fprintf(stderr, "\a=> %s - last change: %s -> %s\n", lptr->name, lptr->date, lstat_t.date);
				syslog(LOG_WARNING, "=> %s - last change: %s -> %s\n", lptr->name, lptr->date, lstat_t.date);
				fprintf(logfd, "=> %s - last change: %s -> %s\n", lptr->name, lptr->date, lstat_t.date);
			}

			if (strcmp(lptr->md5msg, lstat_t.md5msg) != 0) {
				fprintf(stderr, "\a=> %s - md5 digest: %s -> %s\n", lptr->name, lptr->md5msg, lstat_t.md5msg);
				syslog(LOG_WARNING, "=> %s - md5 digest: %s -> %s\n", lptr->name, lptr->md5msg, lstat_t.md5msg);
				fprintf(logfd, "=> %s - md5 digest: %s -> %s\n", lptr->name, lptr->md5msg, lstat_t.md5msg);
			}
			return ret;
		}	
	lptr++; // other data in memory
	}
	return ret;
}

void carregar(int pos) {
	char *pbuffer;
	int cnt = 0;

	lptr = &lstat_p[0];
	
	if ((pbuffer = malloc(MAXBUFF)) == NULL) { 
		perror("malloc()");
		exit(-1);
	}	

	fseek(pfile, pos, SEEK_SET);

//	for(cnt = 0; cnt <  TOTLINE; cnt++) { seg fault
	while((cnt < TOTLINE) && (!feof(pfile))) {
		bzero(pbuffer, MAXBUFF);
		fscanf(pfile, "%[^\n]\n", pbuffer);

		if ( strlen(pbuffer) == 0 ) 
			continue;

		sprintf(lptr->name, "%s", strtok(pbuffer, "|"));
		lptr->uid = atoi(strtok(NULL, "|"));
		lptr->gid = atoi(strtok(NULL, "|"));
		lptr->size = atoi(strtok(NULL, "|"));
		lptr->perm = atoi(strtok(NULL, "|"));
		sprintf(lptr->date, "%s", strtok(NULL, "|"));
		sprintf(lptr->md5msg, "%s", strtok(NULL, "\n"));
		cnt++;
		lptr++;
	}				

	pos = ftell(pfile);

	free(pbuffer); 
        lptr = &lstat_p[0];
}
		
int verify1() {
	unsigned int count = 0;
	unsigned short ret;

	if ( ( pfile = fopen(PFILE, "r") ) == NULL ) {
		fprintf(stderr, "Cannot open %s : %s\n", PFILE, strerror(errno)); 
		return 1;
	}
	
	if ( ( tfile = fopen(TFILE, "r") ) == NULL ) {
		fprintf(stderr, "Cannot open %s : %s\n", TFILE, strerror(errno));
		return 1;
	}

/*
	if ( ( logfd = fopen(LSTATLOG, "a+") ) == NULL ) {
		fprintf(stderr, "Cannot open %s : %s\n", LSTATLOG, strerror(errno));
		return -1;
	}

	fseek(logfd, 0, SEEK_END);
*/
	fprintf(logfd, "\n");

	(void) carregar(pos); 


	fprintf(stderr, "\n");

	while(!feof(tfile)) {
		fscanf(tfile, "%[^\n]\n", tbuffer);	

		if(count == TOTLINE) { //isso vai sair daki... nao vai mais
			(void) carregar(pos);
			count = 0;
		}

		count++;
		
		sprintf(lstat_t.name, "%s", strtok(tbuffer, "|"));
		lstat_t.uid = atoi(strtok(NULL, "|"));
		lstat_t.gid = atoi(strtok(NULL, "|"));
		lstat_t.size = atoi(strtok(NULL, "|"));
		lstat_t.perm = atoi(strtok(NULL, "|"));
		sprintf(lstat_t.date, "%s", strtok(NULL, "|"));
		sprintf(lstat_t.md5msg, "%s", strtok(NULL, "\n"));
	
		/* okay, verify now */	
		ret = memview();
		
		switch(ret) {
			case 1: (void) diskview(); 
				/* search at the disk */
				break;
			case 0: break;
		}
	}

	fclose(tfile); fclose(pfile);
	return 0;
}				
